<?php

namespace DMG\Wires;

if ( class_exists( 'WPCOM_VIP_CLI_Command' ) ) :

/**
 * Commands for DMG Wires
 *
 * @package DMG\Wires
 */
class CLI extends \WPCOM_VIP_CLI_Command {

	/**
	 * Backfill existing Wires with post meta for their converted posts
	 * 
	 * These can be retrieved using a meta query but better practice would
	 * be being able to retrieve them via post meta
	 *
	 * @param array $args       Command args
	 * @param array $assoc_args Command associated args
	 *
	 * @subcommand backfill-converted-post-meta
	 *
	 * @synopsis [--date-after=<date>] [--date-before=<date>] [--dry-run]
	 *
	 * ## EXAMPLES
	 *
	 *  wp dmg-wires backfill-converted-post-meta --date-after=2022-01-15  --date-before=2022-03-15 --dry-run=true
	 */
	public function backfill_converted_post_meta( $args, $assoc_args ) {

		$this->start_bulk_operation();

		// Default to a dry-run
		$dry_run = isset( $assoc_args['dry-run'] ) ? filter_var( $assoc_args['dry-run'], FILTER_VALIDATE_BOOLEAN ) : false;
		if ( $dry_run ) {
			\WP_CLI::log( 'Doing a dry run. No data will be updated.' );
		}

		// Set date query parameters
		$date_after  = ( isset( $assoc_args['date-after'] ) ) ? strtotime( $assoc_args['date-after'] ) : gmdate( 'U' );
		$date_before = ( isset( $assoc_args['date-before'] ) ) ? strtotime( $assoc_args['date-before'] ) : gmdate( 'U' );

		// Define page param
		$page = 1;

		// Counter
		$counter = 0;

		do {

			// Query for all published (i.e. converted) Wires posts
			$wires_query = new \WP_Query(
				array(
					'post_type'      => Post_Type::NAME,
					'post_status'    => 'publish',
					'posts_per_page' => 1,
					'paged'          => $page,
					'date_query'    => array(
						array(
							'inclusive' => true,
							'after'     => array(
								'year'  => gmdate( 'Y', $date_after ),
								'month' => gmdate( 'm', $date_after ),
								'day'   => gmdate( 'd', $date_after ),
							),
							'before'    => array(
								'year'  => gmdate( 'Y', $date_before ),
								'month' => gmdate( 'm', $date_before ),
								'day'   => gmdate( 'd', $date_before ),
							),
						),
					),
				)
			);

			if ( $wires_query->have_posts() ) {

				foreach ( $wires_query->posts as $wire_post ) {

					\WP_CLI::line( "Checking Wires ID {$wire_post->ID}..." );

					$converted_posts = get_post_meta( $wire_post->ID, Post_Type::META_KEY_CONVERTED_POST );

					// Query for all posts with the source wire set as the current post
					$converted_query = new \WP_Query(
						array(
							'post_type'      => 'post',
							'post_status'    => 'any',
							'posts_per_page' => 100, // Current relationship is 1:1 so this is more than enough
							'meta_query'     => array(
								array(
									'key'   => Post_Type::META_KEY_SOURCE_POST,
									'value' => $wire_post->ID
								)
							)
						)
					);

					if ( $converted_query->have_posts() ) {
						
						foreach ( $converted_query->posts as $converted_post ) {

							// Converted post already exists in post meta; skip it
							if ( in_array( $converted_post->ID, $converted_posts ) ) {
								\WP_CLI::line( "Post ID {$converted_post->ID} already exists as converted post in post meta. Skipping..." );
								continue;
							}

							// Add post meta
							if ( ! $dry_run ) {
								add_post_meta( $wire_post->ID, Post_Type::META_KEY_CONVERTED_POST, $converted_post->ID );
								\WP_CLI::line( "Post ID {$converted_post->ID} added as converted post to Wires ID {$wire_post->ID}." );
							} else {
								\WP_CLI::line( "Post ID {$converted_post->ID} will be added as converted post to Wires ID {$wire_post->ID}." );
							}

							// Increment counter
							$counter++;
						}
					}

					\WP_CLI::line( '' );
				}
			}

			// Increase the page count
			$page ++;

			// Free up memory
			$this->vip_inmemory_cleanup();

			sleep( 1 );

		} while ( $wires_query->have_posts() );

		if ( $counter === 0 ) {
			\WP_CLI::line( 'No wires were updated with converted post data.' );
		} else {
			if ( $dry_run ) {
				\WP_CLI::line( "{$counter} posts will be added as converted post data to their parent wires." );
			} else {
				\WP_CLI::line( "{$counter} posts were added as converted post data to their parent wires." );
			}
		}

	}
}

\WP_CLI::add_command( 'dmg-wires', 'DMG\Wires\CLI' );

endif;
