<?php

namespace DMG\Wires;

/**
 * Class Post_Type
 *
 * Register a new wires post type
 *
 * @package DMG\Wires
 */
class Post_Type {

	/**
	 * Post type name
	 *
	 * @var string
	 */
	const NAME = 'dmg_wire';

	/**
	 * Capability name
	 * 
	 * @var string
	 */
	const CAPABILITY = 'wires_editor';

	/**
	 * Meta key for storing the source service ID
	 * 
	 * @var string
	 */
	const META_KEY_SOURCE_ID = 'dmg_wires_source_id';

	/**
	 * Meta key for posts for storing the source wire ID
	 * 
	 * @var string
	 */
	const META_KEY_SOURCE_POST = 'dmg_wires_source_post';

	/**
	 * Meta key for storing converted post ID
	 * 
	 * @var string
	 */
	const META_KEY_CONVERTED_POST = 'dmg_wires_converted_post';

	/**
	 * Meta key for word count
	 * 
	 * @var string
	 */
	const META_KEY_WORD_COUNT = 'dmg_wires_word_count';

	/**
	 * The name for the post conversion action
	 * 
	 * @var string
	 */
	const ACTION_POST_CONVERSION = 'dmg_wires_post_conversion';

	/**
	 * The name for the bulk post conversation action
	 * 
	 * @var string
	 */
	const ACTION_BULK_POST_CONVERSION = 'dmg_wires_bulk_post_conversion';

	/**
	 * Init
	 */
	public static function init() {
		add_action( 'init', array( __CLASS__, 'register' ) );
		add_action( 'admin_init', array( __CLASS__, 'add_capability' ) );
		add_action( 'admin_menu', array( __CLASS__, 'remove_submenu_items' ) );
		add_action( 'load-edit.php', array( __CLASS__, 'add_edit_hooks' ) );
		add_action( 'admin_action_' . self::ACTION_POST_CONVERSION, array( __CLASS__, 'single_convert_to_post' ) );
		add_action( 'add_meta_boxes', array( __CLASS__, 'add_source_text_meta_box' ) );
		add_action( 'add_meta_boxes_' . self::NAME, array( __CLASS__, 'add_converted_posts_list_meta_box' ) );
		add_filter( 'handle_bulk_actions-edit-' . self::NAME, array( __CLASS__, 'handle_bulk_convert' ), 10, 3 );
		add_action( 'enqueue_block_editor_assets', array( __CLASS__, 'enqueue_block_editor_assets' ) );
		add_action( 'wp_before_admin_bar_render', array( __CLASS__, 'remove_admin_bar_link' ) );
	}

	/**
	 * Register the post type
	 */
	public static function register() {
		register_post_type(
			self::NAME,
			array(
				'label'        => 'Wires',
				'labels'       => array(
					'name'          => 'Wires',
					'singular_name' => 'Wire',
					'not_found'     => 'No wires found',
				),
				'description'  => 'Post type for storing ingested wires',
				'public'       => false,
				'hierarchical' => false,
				'show_in_rest' => true,
				'show_ui'      => true,
				'show_in_menu' => true,
				'menu_icon'    => 'dashicons-superhero',
				'supports'     => array(
					'title',
					'editor',
					'excerpt',
					'thumbnail',
					'custom-fields',
				),
				'taxonomies'   => array(
					'post_tag',
					'category',
				),
				'map_meta_cap' => true,
				'capabilities' => array(
					'read'                   => self::CAPABILITY,
					'create_posts'           => self::CAPABILITY,
					'edit_posts'             => self::CAPABILITY,
					'delete_posts'           => self::CAPABILITY,
					'publish_posts'          => self::CAPABILITY,
					'edit_published_posts'   => self::CAPABILITY,
					'delete_published_posts' => self::CAPABILITY,
					'edit_others_posts'      => self::CAPABILITY,
					'delete_others_posts'    => self::CAPABILITY,
					'read_private_posts'     => self::CAPABILITY,
					'edit_private_posts'     => self::CAPABILITY,
					'delete_private_posts'   => self::CAPABILITY,
				)
			)
		);
	}

	/**
	 * Add the capability for administrators
	 */
	public static function add_capability() {
		$role = get_role( 'administrator' );
		$role->add_cap( self::CAPABILITY );
	}

	/**
	 * Remove submenu item for Add New
	 */
	public static function remove_submenu_items() {
		remove_submenu_page(
			'edit.php?post_type=' . self::NAME,
			'post-new.php?post_type=' . self::NAME
		);
	}

	/**
	 * Add edit.php hooks
	 */
	public static function add_edit_hooks() {

		// Make sure we're on the wires edit.php page
		if ( empty( $_GET['post_type'] )
			|| $_GET['post_type'] !== self::NAME
		) {
			return;
		}

		// Filter nomenclature: the post status "Pending" is a little misleading;
		// for the purposes of this plugin, we will map the name of the "Pending"
		// status for wires to "Awaiting Review"
		add_action( 'pre_get_posts', array( __CLASS__, 'filter_query' ) );
		add_filter( 'views_edit-' . self::NAME, array( __CLASS__, 'filter_views' ) );
		add_filter( 'display_post_states', array( __CLASS__, 'filter_post_states' ), 10, 2 );
		add_action( 'add_inline_data', array( __CLASS__, 'add_inline_excerpt' ) );
		add_action( 'admin_enqueue_scripts', array( __CLASS__, 'enqueue_assets' ) );
		add_filter( 'post_row_actions', array( __CLASS__, 'add_convert_link' ), 10, 2 );
		add_filter( 'bulk_actions-edit-' . self::NAME, array( __CLASS__, 'add_bulk_convert_action' ) );
		add_action( 'admin_notices', array( __CLASS__, 'add_notices' ) );
		add_action( 'restrict_manage_posts', array( __CLASS__, 'add_filters' ) );
		add_filter( 'manage_' . self::NAME . '_posts_columns', array( __CLASS__, 'filter_columns' ) );
		add_filter( 'manage_' . self::NAME . '_posts_custom_column', array( __CLASS__, 'add_word_count_column' ), 10, 2 );
		add_filter( 'post_date_column_time', array( __CLASS__, 'filter_modified_time' ), 10, 2 );

		// Default to 100 posts per page on the list view
		add_filter( 'edit_posts_per_page', function() {
			return 100;
		} );
	}

	/**
	 * Filter edit.php query
	 * 
	 * @param WP_Query $query The query object
	 */
	public static function filter_query( $query ) {

		// Order by date in descending order
		if ( isset( $query->query['post_status'] )
			&& 'pending' === $query->query['post_status']
		) {
			$query->set( 'order', 'DESC' );
			$query->set( 'orderby', 'modified' );
		}

		// Set the date, if specified
		if ( ! empty( $_GET['dmg-wires-date'] ) ) {
			$date   = sanitize_text_field( $_GET['dmg-wires-date'] );
			$pieces = explode( '-', $date );
			if ( ! empty( $pieces[0] )
				&& ! empty( $pieces[1] )
				&& ! empty( $pieces[2] )
			) {
				$query->set( 'date_query', array(
					array(
						'year'  => (int) $pieces[0],
						'month' => (int) $pieces[1],
						'day'   => (int) $pieces[2]
					)
				) );
			}
		}
	}

	/**
	 * Filter the views
	 * 
	 * @param array $views The current views
	 * @return array The filtered views
	 */
	public static function filter_views( $views ) {

		// Rename "Pending" to "Pending Review"
		if ( isset( $views['pending'] ) ) {
			$views['pending'] = str_replace( 'Pending', 'Pending Review', $views['pending'] );
		}

		// Rename "Published" to "Converted"
		if ( isset( $views['publish'] ) ) {
			$views['publish'] = str_replace( 'Published', 'Converted', $views['publish'] );
		}

		// Remove "Mine"
		if ( isset( $views['mine'] ) ) {
			unset( $views['mine'] );
		}
		
		return $views;
	}

	/**
	 * Filter post states
	 * 
	 * @param array   $states The existing post states
	 * @param WP_Post $post   The post object
	 * @return array The filtered states
	 */
	public static function filter_post_states( $states, $post ) {
		
		// Post type check
		if ( ! isset( $post->post_type )
			|| $post->post_type !== self::NAME
		) {
			return $states;
		}

		// Rename "Pending" to "Pending Review"
		if ( isset( $states['pending'] ) ) {
			$states['pending'] = 'Pending Review';
		}

		// Rename "Published" to "Converted"
		if ( isset( $states['publish'] ) ) {
			$states['publish'] = 'Converted';
		}

		return $states;
	}

	/**
	 * Add the excerpt to the hidden inline data (first two paragraphs)
	 * 
	 * @param WP_Post $post The post object
	 */
	public static function add_inline_excerpt( $post ) {
		$blocks = parse_blocks( $post->post_content );
		$content = '';
		$count = 0;
		foreach ( $blocks as $block ) {
			if ( ! isset( $block['blockName'] )
				|| $block['blockName'] !== 'core/paragraph'
			) {
				continue;
			}

			$content .= $block['innerHTML'];

			$count++;

			if ( $count === 2 ) {
				break;
			}
		}
		echo '<div class="dmg-wires__inline-excerpt">' . wp_kses_post( $content ) . '</div>';
	}

	/**
	 * Enqueue some assets for edit.php
	 */
	public static function enqueue_assets() {

		// Styles for edit.php
		wp_enqueue_style(
			'dmg-wires-edit-php',
			plugin_dir_url( __FILE__ ) . '/css/edit-php.css',
			array(),
			DMG_WIRES_VERSION
		);

		// Enqueue WP API Request
		wp_enqueue_script( 'wp-api-request' );

		// Tag filter for edit.php
		wp_enqueue_script(
			'dmg-wires-tag-filter',
			plugin_dir_url( __FILE__ ) . '/js/tag-filter.js',
			array(),
			DMG_WIRES_VERSION
		);
	}

	/**
	 * Add link to Convert to Post (for pending wires) or Edit Converted Post
	 * (for converted wires)
	 * 
	 * @param array   $actions The current post row actions
	 * @param WP_Post $post    The post object
	 * @return array The filtered post row actions
	 */
	public static function add_convert_link( $actions, $post ) {

		// Create the Convert to Post URL
		$admin_url = admin_url(
			sprintf(
				'admin.php?action=%s&post=%s&_wpnonce=%s',
				self::ACTION_POST_CONVERSION,
				$post->ID,
				wp_create_nonce( self::ACTION_POST_CONVERSION )
			)
		);

		// Add option to Convert to New Post and Edit Converted Post
		if ( 'publish' === $post->post_status
			&& ! empty( $post->post_parent )
		) {
			// Add a Edit Converted Post action at the beginning of the post row
			$actions = array(
				'dmg-wires__convert-to-new-post' => sprintf(
					'<a href="%s">Convert to New Post</a>',
					esc_url( $admin_url )
				)
			) + array(
				'dmg-wires__edit-converted-post' => sprintf(
					'<a href="%s">Edit Latest Converted Post</a>',
					esc_url( get_edit_post_link( $post->post_parent ) )
				)
			) + $actions;

		} else if ( 'pending' === $post->post_status ) {

			// Add a Convert to Post action at the beginning of the post row
			$actions = array(
				'dmg-wires__convert-to-post' => sprintf(
					'<a href="%s">Convert to Post</a>',
					esc_url( $admin_url )
				)
			) + $actions;
		}

		return $actions;
	}

	/**
	 * Post conversion callback
	 */
	public static function single_convert_to_post() {
		
		if ( ! isset( $_GET['_wpnonce'] ) ) {
			wp_die( 'No nonce set.' );
		}

		$nonce = sanitize_text_field( $_GET['_wpnonce'] );
		if ( ! wp_verify_nonce( $nonce, self::ACTION_POST_CONVERSION ) ) {
			wp_die( "Couldn't verify nonce." );
		}

		if ( empty( $_GET['post'] ) ) {
			wp_die( 'No post specified' );
		}

		$post_id = self::convert_to_post( (int) $_GET['post'] );

		// Redirect if we received a post ID
		if ( is_int( $post_id ) ) {

			$redirect_url = admin_url(
				sprintf(
					'post.php?post=%s&action=edit',
					$post_id
				)
			);

			// Redirect to the new post
			wp_redirect( $redirect_url );
		}
	}

	/**
	 * Convert a single wire to a post
	 * 
	 * @param int $source_id The source wire ID
	 * @return int|bool The converted post ID or false if something went wrong
	 */
	public static function convert_to_post( $source_id ) {

		$post = get_post( $source_id );
		if ( ! is_a( $post, 'WP_Post' ) ) {
			return false;
		}

		/**
		 * Filter the post data before inserting
		 * 
		 * @param array   $postdata The post data to insert
		 * @param WP_Post $post     The post object
		 */
		$postdata = apply_filters(
			'dmg_wires_conversion_postdata',
			array(
				'post_type'    => 'post',
				'post_title'   => $post->post_title,
				'post_content' => $post->post_content,
				'post_excerpt' => $post->post_excerpt,
				'post_date'    => $post->post_date,
				'post_status'  => 'draft',
			),
			$post
		);

		$post_id = wp_insert_post( $postdata );

		// If successful, set some additional info
		if ( is_wp_error( $post_id ) ) {
			wp_die( 'Something went wrong during post conversation.' );
		}

		// Copy over tags and categories
		foreach ( array( 'post_tag', 'category' ) as $taxonomy ) {
			$terms = wp_get_object_terms( $source_id, $taxonomy );
			if ( is_array( $terms ) ) {
				$terms = array_map( function( $term ) {
					return $term->term_id;
				}, $terms );

				wp_set_object_terms( $post_id, $terms, $taxonomy );
			}
		}

		// Store source in meta
		update_post_meta( $post_id, self::META_KEY_SOURCE_POST, $source_id );

		// Store converted post ID in Wires meta
		add_post_meta( $source_id, self::META_KEY_CONVERTED_POST, $post_id );

		// Update the source object parent and post status
		wp_update_post(
			array(
				'ID'          => $source_id,
				'post_parent' => $post_id,
				'post_status' => 'publish',
			)
		);

		// Fire a subsequent update request so the source wire retains it's original
		// post date. The above publish action will set post_date to the current date,
		// even when we specify it
		wp_update_post(
			array(
				'ID'        => $source_id,
				'post_date' => $post->post_date,
			)
		);

		// Clone images in converted post and set appropriate replacements
		$converted_post = get_post( $post_id );

		if ( has_block( 'core/image', $converted_post ) ) {

			// Parse out blocks for rejoining later
			$blocks = parse_blocks( $converted_post->post_content );

			foreach ( $blocks as &$block ) {

				// Seek out the image blocks
				if ( ! isset( $block['blockName'] )
					|| 'core/image' !== $block['blockName']
					|| empty( $block['attrs']['id'] )
				) {
					continue;
				}

				// Get original source image for sideloading
				$original_source = get_post_meta( $block['attrs']['id'], '_source_url', true);
				if ( empty( $original_source ) ) {
					continue;
				}

				// Sideload image to clone it
				$image_id = media_sideload_image(
					$original_source,
					$post_id, // Associate with the newly converted post
					null,
					'id'
				);

				// Retrieve full source of cloned image
				$src = wp_get_attachment_image_src( $image_id, 'full' );

				// Get metadata for populating block
				$metadata = wp_get_attachment_metadata( $image_id );

				// Create a new images block to replace the old one
				if ( is_int( $image_id )
					&& ! empty( $src[0] )
				) {
					// Drop new attachment meta for source attachment and parent post ID
					update_post_meta( $image_id, self::META_KEY_SOURCE_POST, $block['attrs']['id'] );
					update_post_meta( $image_id, 'dmg_wires_parent_post_id', $post_id );

					// Save the original wires guid, as well
					$guid = get_post_meta( $source_id, self::META_KEY_SOURCE_ID, true );
					if ( ! empty( $guid ) ) {
						update_post_meta( $image_id, self::META_KEY_SOURCE_ID, $guid );
					}

					$caption = ( ! empty( $metadata['image_meta']['credit'] ) ) ? $metadata['image_meta']['credit'] : '';

					// Build out the block attributes
					$json = array(
						'url'             => $src[0],
						'caption'         => ! empty( $caption ) ? $caption : '',
						'id'              => (int) $image_id,
						'sizeSlug'        => 'full',
						'linkDestination' => 'none',
					);

					// Build out the block content
					if ( ! empty( $caption ) ) {

						// When caption is set, include the [caption] shortcode
						$shortcode = sprintf(
							'[caption align="alignnone" attachment_id="%d" id="attachment_%d" width="%d" ]<img src="%s" class="wp-image-%d" width="%d" height="%d"/> %s[/caption]',
							(int) $image_id,
							(int) $image_id,
							(int) $metadata['width'],
							str_replace( '&', '\u0026', $src[0] ),
							(int) $image_id,
							(int) $metadata['width'],
							(int) $metadata['height'],
							$caption
						);
					} else {

						// When caption is absent, just use the <img /> tag
						$shortcode = sprintf(
							'<img src="%s" class="wp-image-%d" width="%d" height="%d"/>',
							str_replace( '&', '\u0026', $src[0] ),
							(int) $image_id,
							(int) $metadata['width'],
							(int) $metadata['height']
						);
					}

					// Reconfigure the block before serializing
					$block['attrs']['id']      = (int) $image_id;
					$block['attrs']['caption'] = $caption;
					$block['attrs']['url']     = $src[0];
					$block['innerHTML']        = "\n" . $shortcode . "\n";
					$block['innerContent']     = array(
						$block['innerHTML']
					);
				}
			}

			// Update post content with new content string containing cloned images
			wp_update_post(
				array(
					'ID'           => $post_id,
					'post_content' => serialize_blocks( $blocks )
				)
			);
		}

		/**
		 * Fire an action after conversation complete
		 * 
		 * @param int $post_id   The newly created post ID
		 * @param int $source_id The post ID of the source wire
		 */
		do_action( 'dmg_wires_converted_post', $post_id, $source_id );

		return $post_id;
	}

	/**
	 * Add source text meta box
	 */
	public static function add_source_text_meta_box() {
		add_meta_box(
			'dmg_wires_post_source_text',
			'Wire Source Text',
			array( __CLASS__, 'source_text_meta_box_callback' ),
			'post',
			'normal',
			'high'
		);
	}

	/**
	 * Source text meta box callback
	 * 
	 * @param WP_Post $post The editor post object
	 */
	public static function source_text_meta_box_callback( $post ) {
		
		// Get source post ID
		$source_post_id = get_post_meta( $post->ID, self::META_KEY_SOURCE_POST, true );
		if ( empty( $source_post_id ) ) {
			return;
		}

		$source_post = get_post( $source_post_id );
		if ( empty( $source_post->post_content ) ) {
			return;
		}

		$service = 'No service found';
		$services = get_the_terms( $source_post_id, Taxonomy::NAME );
		if ( ! empty( $services[0]->name ) ) {
			$service = $services[0]->name;
		}

		// Generate the source wire link
		$source_link = sprintf(
			'<a href="%s" target="_blank">View source wire</a>',
			esc_url( get_edit_post_link( $source_post_id ) )
		);

		printf(
			'<h3 style="display:inline">%s (%s) </h3>%s%s',
			get_the_title( $source_post ),
			$service,
			( current_user_can( self::CAPABILITY ) ) ? $source_link : '',
			apply_filters( 'the_content', $source_post->post_content )
		);
	}

	/**
	 * Add converted posts list meta box to Wire post type
	 */
	public static function add_converted_posts_list_meta_box() {

		// Check for converted posts before adding the meta box
		$converted_posts = self::get_converted_posts( get_the_ID() );
		if ( empty( $converted_posts ) ) {
			return;
		}

		add_meta_box(
			'dmg_wires_converted_posts_list',
			'Converted Posts',
			array( __CLASS__, 'converted_posts_list_meta_box_callback' ),
			self::NAME,
			'normal',
			'high'
		);
	}

	/**
	 * Callback for converted posts list meta box
	 * 
	 * @param WP_Post $post The editor post object
	 */
	public static function converted_posts_list_meta_box_callback( $post ) {
		
		// Get converted posts
		$posts = self::get_converted_posts( $post->ID );

		// No posts found; early exit
		if ( empty( $posts ) ) {
			printf( '<p>No posts found.</p>' );
			return;
		}

		// Print out each post
		foreach ( $posts as $post ) {
			printf(
				'<div class="dmg-wires-post">
					<h3><a href="%s" target="_blank">%s</a> – <span class="dmg-wires-post__status">%s</span></h3>
					<h4>%s</h4>
					<p>%s</p>
				</div>',
				get_edit_post_link( $post->ID ),
				get_the_title( $post->ID ),
				ucfirst( $post->post_status ),
				get_the_modified_date( 'F j, Y g:i a', $post->ID ),
				esc_html( $post->post_excerpt )
			);
		}
	}

	/**
	 * Helper for getting converted posts
	 * 
	 * @param int $post_id The wire post ID to get converted posts for
	 * @return array An array of post objects
	 */
	public static function get_converted_posts( $post_id ) {

		static $posts;
		if ( ! is_null( $posts ) ) {
			return $posts;
		}

		$posts = array();

		// Get converted post IDs
		$converted_ids = (array) get_post_meta( $post_id, self::META_KEY_CONVERTED_POST );

		if ( ! empty( $converted_ids ) ) {

			// Query for all posts with the source wire set as the current post
			$query = new \WP_Query(
				array(
					'post_type'      => 'post',
					'post__in'       => $converted_ids,
					'posts_per_page' => count( $converted_ids ),
					'orderby'        => 'modified',
				)
			);

			if ( $query->have_posts() ) {
				$posts = $query->posts;
			}
		}

		return $posts;
	}

	/**
	 * Add bulk action to convert posts
	 * 
	 * @param array $actions The current bulk edit actions
	 * @return array The filtered array of actions
	 */
	public static function add_bulk_convert_action( $actions ) {
		$new_actions = array(
			self::ACTION_BULK_POST_CONVERSION => 'Convert to Post'
		);

		return $new_actions + $actions;
	}

	/**
	 * Handle bulk convert
	 * 
	 * @param string $sendback The redirect URL
	 * @param string $doaction The action
	 * @param array  $items    The array of post IDs being converted
	 * @return array The filtered redirect URL
	 */
	public static function handle_bulk_convert( $sendback, $doaction, $items ) {

		// Check action name
		if ( self::ACTION_BULK_POST_CONVERSION === $doaction ) {

			$converted_ids = array();

			// Perform conversion on each item
			foreach ( $items as $item ) {
				$converted_id = self::convert_to_post( $item );
				if ( is_int( $converted_id ) ) {
					$converted_ids[] = $converted_id;
				}
			}

			$sendback = add_query_arg( 'dmg-wires-converted', count( $converted_ids ), $sendback );
		}

		return $sendback;
	}

	/**
	 * Add notices showing results of bulk conversion
	 */
	public static function add_notices() {

		if ( isset( $_GET['dmg-wires-converted'] ) ) {

			$converted = (int) $_GET['dmg-wires-converted'];

			printf(
				'<div id="message" class="updated %s is-dismissable"><p>%s</p></div>',
				( $converted === 0 ) ? 'error' : 'notice',
				( $converted === 0 ) ? 'No wires were converted!' : 'Successfully converted ' . $converted . ' wires.'
			);
		}
	}

	/**
	 * Add filters for date, tag, and source
	 */
	public static function add_filters() {
		$date   = ( ! empty( $_GET['dmg-wires-date'] ) ) ? sanitize_text_field( $_GET['dmg-wires-date'] ) : '';
		$tag    = ( ! empty( $_GET['tag'] ) ) ? sanitize_text_field( $_GET['tag'] ) : '';
		$source = ( ! empty( $_GET[ Taxonomy::NAME ] ) ) ? sanitize_text_field( $_GET[ Taxonomy::NAME ] ) : '';

		$term = ( ! empty( $tag ) ) ? get_term_by( 'slug', $tag, 'post_tag' ) : null;
		
		$template = sprintf(
			'<span class="dmg-wires__tag">
				<span class="dashicons dashicons-no-alt"></span>
				<span class="dmg-wires__tag-name">%s</span>
			</span>',
			( is_a( $term, 'WP_Term' ) ) ? $term->name : '',
		);

		// Fetch all sources
		$sources = get_terms(
			array(
				'taxonomy'   => Taxonomy::NAME,
				'hide_empty' => false,
			)
		);

		// Build options HTML
		$source_html = '';
		foreach ( $sources as $source_term ) {
			if ( is_a( $source_term, 'WP_Term' ) ) {
				$source_html .= sprintf(
					'<option value="%s" %s>%s</option>',
					$source_term->slug,
					selected( $source_term->slug, $source, false ),
					$source_term->name
				);
			}
		}

		// Output markup selected
		$selected = '';
		if ( is_a( $term, 'WP_Term' ) ) {
			// Output the selected 
			$selected = $template;
		}
		?>
		<select name=<?php echo esc_attr( Taxonomy::NAME ); ?>>
			<option value="">All Wire Sources</option>
			<?php
			echo wp_kses(
				$source_html,
				array(
					'option' => array(
						'value'    => array(),
						'selected' => array(),
					)
				)
			); ?>
		</select>
		<input type="date" name="dmg-wires-date" value="<?php echo esc_attr( $date ); ?>" />
		<div class="dmg-wires__tag-selector <?php echo ( ! empty( $selected ) ) ? 'dmg-wires__tag-selector--selected' : ''; ?>">
			<template>
				<?php echo wp_kses_post( $template ); ?>
			</template>
			<input type="text" placeholder="Search tags" name="dmg-wires-tag" value="" />
			<input type="hidden" name="tag" value="<?php echo esc_attr( $tag ); ?>" />
			<span class="spinner"></span>
			<div class="dmg-wires__tag-results"></div>
			<?php echo wp_kses_post( $selected ); ?>
		</div>
		<?php
	}

	/**
	 * Add "Word Count" column
	 * 
	 * @param array $columns The existing columns
	 * @return array The filtered columns
	 */
	public static function filter_columns( $columns ) {
		$columns['dmg_wires_word_count'] = 'Word Count';

		return $columns;
	}

	/**
	 * Add word count to post row
	 * 
	 * @param string $column  The column name
	 * @param int    $post_id The post ID
	 */
	public static function add_word_count_column( $column, $post_id ) {
		if ( 'dmg_wires_word_count' === $column ) {
			echo esc_html( get_post_meta( $post_id, Post_Type::META_KEY_WORD_COUNT, true ) );
		}
	}

	/**
	 * Filter the date column to show modified
	 * 
	 * @param string  $date The current date
	 * @param WP_Post $post The post object
	 * @return string The filtered date string
	 */
	public static function filter_modified_time( $date, $post ) {

		// If pending, set the post modified date as the display date
		if ( isset( $post->post_status )
			&& 'pending' === $post->post_status
		) {
			// Match the date format from core
			$date = sprintf(
				'%1$s at %2$s',
				get_the_modified_date( 'Y/m/d', $post ),
				get_the_modified_date( 'g:i a', $post )
			);
		}

		return $date;
	}

	/**
	 * Enqueue block editor assets
	 */
	public static function enqueue_block_editor_assets() {

		if ( self::NAME !== get_post_type() ) {
			return;
		}

		// Styles for edit.php
		wp_enqueue_style(
			'dmg-wires-post-php',
			plugin_dir_url( __FILE__ ) . '/css/post-php.css',
			array(),
			DMG_WIRES_VERSION
		);
		
		// Add button for converting to post
		$asset = require_once( dirname( __FILE__ ) . '/build/convert-to-post.asset.php' );

		wp_enqueue_script(
			'dmg-wires-convert-to-post',
			plugin_dir_url( __FILE__ ) . '/build/convert-to-post.js',
			$asset['dependencies'],
			$asset['version']
		);

		wp_add_inline_script(
			'dmg-wires-convert-to-post',
			'var DMGWires = {
				nonce: \'' . esc_js( wp_create_nonce( self::ACTION_POST_CONVERSION ) ) . '\'
			}',
			'before'
		);
	}

	/**
	 * Remove admin bar link
	 */
	public static function remove_admin_bar_link() {
		global $wp_admin_bar;
		$wp_admin_bar->remove_menu( 'new-dmg_wire' );
	}
}

Post_Type::init();
