<?php

namespace DMG\Wires\Services;

/**
 * Class PA_Media
 *
 * Register the PA Media service
 *
 * @package DMG\Wires\Services
 */
class PA_Media extends Service {

	/**
	 * The name of the API key option
	 * 
	 * @var string
	 */
	const OPTION_NAME_API_KEY = 'dmg_wires_pa_media_api_key';

	/**
	 * The name of the Enable Collation option
	 * 
	 * @var string
	 */
	const OPTION_NAME_ENABLE_COLLATION = 'dmg_wires_pa_media_enable_collation';

	/**
	 * The name of the settings section
	 * 
	 * @var string
	 */
	const SETTINGS_SECTION = 'dmg_wires_pa_media';

	/**
	 * API endpoint
	 * 
	 * @var string
	 */
	const API_ENDPOINT = 'https://content.api.pressassociation.io/v1/item';

	/**
	 * Source term name
	 * 
	 * @var string
	 */
	const SOURCE_NAME = 'PA Media';

	/**
	 * Meta key for event ID
	 * 
	 * @var string
	 */
	const META_KEY_EVENT_ID = 'dmg_wires_pa_media_event_id';

	/**
	 * Constructor
	 */
	public function __construct() {
		add_action( 'admin_init', array( $this, 'register_fields' ) );
	}

	/**
	 * Register
	 */
	public function register_fields() {

		// Register a new settings section
		add_settings_section(
			self::SETTINGS_SECTION,
			'PA Media',
			array( $this, 'settings_section_callback' ),
			$this->get_settings_page_name()
		);

		// Register the settings and the corresponding option
		register_setting(
			$this->get_option_group(),
			self::OPTION_NAME_API_KEY,
		);

		// Register API Key field
		add_settings_field(
			self::OPTION_NAME_API_KEY,
			'API Key',
			array( $this, 'api_key_field_callback' ),
			$this->get_settings_page_name(),
			self::SETTINGS_SECTION
		);

		register_setting(
			$this->get_option_group(),
			self::OPTION_NAME_ENABLE_COLLATION,
		);

		// Register Enable Collation field
		add_settings_field(
			self::OPTION_NAME_ENABLE_COLLATION,
			'Enable Collation?',
			array( $this, 'enable_collation_field_callback' ),
			$this->get_settings_page_name(),
			self::SETTINGS_SECTION
		);
	}

	/**
	 * Settings section callback
	 */
	public function settings_section_callback( $args ) {
		echo '<p>Configure settings for the PA Media wire service.</p>';
	}

	/**
	 * API Key field callback function
	 *
	 * @param array $args The field args
	 */
	public function api_key_field_callback( $args ) {
		printf(
			'<input type="password" class="widefat" id="%s" name="%s" value="%s" />',
			esc_attr( self::OPTION_NAME_API_KEY ),
			esc_attr( self::OPTION_NAME_API_KEY ),
			get_option( self::OPTION_NAME_API_KEY )
		);
	}

	/**
	 * Enable Collation field callback function
	 *
	 * @param array $args The field args
	 */
	public function enable_collation_field_callback( $args ) {
		printf(
			'<input type="checkbox" id="%s" name="%s" value="1" %s />',
			esc_attr( self::OPTION_NAME_ENABLE_COLLATION ),
			esc_attr( self::OPTION_NAME_ENABLE_COLLATION ),
			checked( get_option( self::OPTION_NAME_ENABLE_COLLATION ), true, false )
		);
	}

	/**
	 * Ingest the latest posts
	 */
	public function ingest_posts() {

		// Bail early if no API key set
		$api_key = get_option( self::OPTION_NAME_API_KEY );
		if ( empty( $api_key ) ) {
			return;
		}

		/**
		 * Offer filter for API call query args
		 * 
		 * By default, only query items where type=text
		 * 
		 * @param array $query_args The query args for the API request
		 */
		$query_args = apply_filters(
			'dmg_wires_pa_media_api_query_args',
			array(
				'apikey'   => $api_key,
				'product'  => 'paproduct:create',
				'sortType' => 'FirstPublishable',
			)
		);
		
		$url = add_query_arg( $query_args, self::API_ENDPOINT );

		// Don't cache; we will need up-to-date results
		$response = wp_remote_get( $url );
		if ( is_wp_error( $response ) ) {
			return;
		}

		// JSON-decode body
		$body = wp_remote_retrieve_body( $response );

		$json = json_decode( $body );
		if ( ! is_object( $json ) ) {
			return;
		}

		if ( empty( $json->item )
			|| ! is_array( $json->item )
		) {
			return;
		}

		// Check if collation is enabled
		$collation_enabled = (bool) get_option( self::OPTION_NAME_ENABLE_COLLATION );

		// Reverse the list of items so that if collation occurs, the new entries
		// are appended in chronological order (i.e. oldest to newest)
		$items = array_reverse( $json->item );

		foreach ( $items as $item ) {

			// Only permit ingestion of "text" stories right now
			if ( empty( $item->type )
				|| 'text' !== $item->type
			) {
				continue;
			}

			// Check the item URI to see if we've already ingested
			if ( empty( $item->uri )
				|| $this->post_exists( $item->uri )
			) {
				continue;
			}

			// If collation is enabled, fetch a post with the existing event ID and attempt to collate
			if ( $collation_enabled ) {

				// There could be more than one event so attempt collation with each one
				$did_collation = false;

				// Set the event ID(s)
				if ( ! empty( $item->event )
					&& is_array( $item->event )
				) {
					foreach ( $item->event as $event ) {

						// Meta query for the event ID
						$query = new \WP_Query(
							array(
								'post_type'      => \DMG\Wires\Post_Type::NAME,
								'post_status'    => array( 'pending', 'publish' ),
								'posts_per_page' => 1,
								'meta_query'     => array(
									array(
										'key'     => self::META_KEY_EVENT_ID,
										'value'   => $event->code,
										'compare' => '=',
									),
								),
							)
						);

						// Attempt collation on found post object
						if ( ! empty( $query->post )
							&& is_a( $query->post, 'WP_Post' )
						) {
							$did_collation = $this->collate( $item, $query->post );
						}
					}
				}

				// If we performed collation, continue without creating a new post
				if ( $did_collation ) {
					continue;
				}
			}

			$excerpt = '';
			$pieces  = explode( "\n\n", $item->body_text );
			if ( ! empty( $pieces[0] ) ) {
				$excerpt = $pieces[0];
			}

			// Create new post object
			$post_id = wp_insert_post(
				array(
					'post_type'    => \DMG\Wires\Post_Type::NAME,
					'post_title'   => ( ! empty( $item->headline ) ) ? $item->headline : "[UNTITLED] PA Media {$item->uri}",
					'post_content' => $this->get_block_string_from_text( $item->body_text ),
					'post_excerpt' => $excerpt,
					'post_date'    => ( ! empty( $item->versioncreated ) ) ? $item->versioncreated : null,
					'post_status'  => 'pending',
				)
			);

			// If successful, set some additional info
			if ( ! is_wp_error( $post_id ) ) {

				// Set the source ID
				$this->set_source_id( $post_id, $item->uri );

				// Set source term
				$this->set_source_term( $post_id, self::SOURCE_NAME );

				// Set the word count
				$this->set_word_count( $post_id, str_word_count( $item->body_text ) );

				// Set subjects and events
				$this->add_subjects_and_events( $post_id, $item );

				// Perform an action after the wire was created
				$this->do_action_after_create_post( $post_id );
			}
		}
	}

	/**
	 * Collate a PA Media update with an existing object
	 * 
	 * @param object  $item The PA Media item
	 * @param WP_Post $post The post object to collate
	 * @return bool If the collation was successful
	 */
	public function collate( $item, $post ) {

		$collated = false;

		// Require a uri, issued date and body text; require a WP_Post
		if ( empty( $item->uri )
			|| empty( $item->versioncreated )
			|| empty( $item->body_text )
			|| ! is_a( $post, 'WP_Post' )
		) {
			return $collated;
		}

		// Get new text along with a human-readable issued date
		$new_text = sprintf(
			"<!-- wp:paragraph -->\n<p class=\"\"><strong>%s</strong></p>\n<!-- /wp:paragraph -->\n\n",
			date( 'F j, Y G:i:s', strtotime( $item->issued ) )
		);

		$new_text .= $this->get_block_string_from_text( $item->body_text );

		// Append new content
		$new_content = $post->post_content . "\n\n" . $new_text;

		// Update the post object
		$updated = wp_update_post(
			array(
				'ID'            => $post->ID,
				'post_content'  => $new_content,
			)
		);

		// If update was successful, set some additional info
		if ( ! is_wp_error( $updated )
			&& ! empty( $updated )
		) {
			// Update the $collated flag
			$collated = true;

			// Add the new source ID
			add_post_meta( $post->ID, \DMG\Wires\Post_Type::META_KEY_SOURCE_ID, $item->uri );

			// Set a new word count
			$this->set_word_count( $post->ID, str_word_count( wp_strip_all_tags( $new_content ) ) );

			// Set subjects and events
			$this->add_subjects_and_events( $post->ID, $item );

			/**
			 * Perform an action after the wire has been collated
			 * 
			 * @param int $post_id The ID of the newly created wire post
			 */
			do_action( 'dmg_wires_after_collate_post', $post->ID );

			// Clean post cache after making direct SQL query
			clean_post_cache( $post->ID );
		}

		return $collated;
	}

	/**
	 * Helper to add subjects (i.e. tax terms) and events
	 * 
	 * @param int    $post_id The post ID
	 * @param object $item    The PA response object
	 */
	public function add_subjects_and_events( $post_id, $item ) {

		// Loop through all subjects and map them to existing tags/categories
		if ( ! empty( $item->subject )
			&& is_array( $item->subject )
		) {
			$tag_ids      = array();
			$category_ids = array();

			foreach ( $item->subject as $subject ) {

				// Skip a subject that doesn't have a name
				if ( empty( $subject->name ) ) {
					continue;
				}

				$tag = get_term_by( 'name', $subject->name, 'post_tag' );
				if ( is_a( $tag, 'WP_Term' ) ) {
					$tag_ids[] = $tag->term_id;
				}

				$category = get_term_by( 'name', $subject->name, 'category' );
				if ( is_a( $category, 'WP_Term' ) ) {
					$category_ids[] = $category->term_id;
				}
			}

			// Append new tags
			if ( ! empty( $tag_ids ) ) {
				wp_set_object_terms( $post_id, $tag_ids, 'post_tag', true );
			}

			// Append new categories
			if ( ! empty( $category_ids ) ) {
				wp_set_object_terms( $post_id, $category_ids, 'category', true );
			}
		}

		// Set the event ID(s)
		if ( ! empty( $item->event )
			&& is_array( $item->event )
		) {

			// Get currently set events, no need to add new post meta if it's already set
			$events = get_post_meta( $post_id, self::META_KEY_EVENT_ID );

			foreach ( $item->event as $event ) {
				if ( ! empty( $event->code )
					&& ! in_array( $event->code, $events, true )
				) {
					add_post_meta( $post_id, self::META_KEY_EVENT_ID, $event->code );
				}
			}
		}
	}
}
