<?php

namespace DMG\Wires\Services;

/**
 * Class Reuters
 *
 * Register the Reuters service
 *
 * @package DMG\Wires\Services
 */
class Reuters extends Service {

	/**
	 * The name of the client ID option
	 * 
	 * @var string
	 */
	const OPTION_NAME_CLIENT_ID = 'dmg_wires_reuters_client_id';

	/**
	 * The name of the audience option
	 * 
	 * @var string
	 */
	const OPTION_NAME_AUDIENCE = 'dmg_wires_reuters_audience';

	/**
	 * The name of the secret option
	 * 
	 * @var string
	 */
	const OPTION_NAME_SECRET = 'dmg_wires_reuters_secret';

	/**
	 * Option to store the auth token
	 * 
	 * @var string
	 */
	const OPTION_NAME_AUTH_TOKEN = 'dmg_wires_reuters_auth_token';

	/**
	 * The name of the settings section
	 * 
	 * @var string
	 */
	const SETTINGS_SECTION = 'dmg_wires_reuters';

	/**
	 * Auth endpoint
	 * 
	 * @var string
	 */
	const AUTH_ENDPOINT = 'https://auth.thomsonreuters.com/oauth/token';

	/**
	 * API endpoint
	 * 
	 * @var string
	 */
	const API_ENDPOINT = 'https://api.reutersconnect.com/content/graphql';

	/**
	 * Source term name
	 * 
	 * @var string
	 */
	const SOURCE_NAME = 'Reuters';

	/**
	 * Constructor
	 */
	public function __construct() {
		add_action( 'admin_init', array( $this, 'register_fields' ) );
	}

	/**
	 * Register
	 */
	public function register_fields() {

		// Register a new settings section
		add_settings_section(
			self::SETTINGS_SECTION,
			'Reuters',
			array( $this, 'settings_section_callback' ),
			$this->get_settings_page_name()
		);

		// Register the settings and the corresponding option
		register_setting(
			$this->get_option_group(),
			self::OPTION_NAME_CLIENT_ID,
		);

		// Register Client ID field
		add_settings_field(
			self::OPTION_NAME_CLIENT_ID,
			'Client ID',
			array( $this, 'client_id_field_callback' ),
			$this->get_settings_page_name(),
			self::SETTINGS_SECTION
		);

		register_setting(
			$this->get_option_group(),
			self::OPTION_NAME_AUDIENCE,
		);

		// Register Audience field
		add_settings_field(
			self::OPTION_NAME_AUDIENCE,
			'Audience',
			array( $this, 'audience_field_callback' ),
			$this->get_settings_page_name(),
			self::SETTINGS_SECTION
		);

		register_setting(
			$this->get_option_group(),
			self::OPTION_NAME_SECRET,
		);

		// Register Secret field
		add_settings_field(
			self::OPTION_NAME_SECRET,
			'Secret',
			array( $this, 'secret_field_callback' ),
			$this->get_settings_page_name(),
			self::SETTINGS_SECTION
		);
	}

	/**
	 * Settings section callback
	 */
	public function settings_section_callback( $args ) {
		echo '<p>Configure settings for the Reuters wire service.</p>';
	}

	/**
	 * Client ID field callback function
	 *
	 * @param array $args The field args
	 */
	public function client_id_field_callback( $args ) {
		printf(
			'<input type="text" class="widefat" id="%s" name="%s" value="%s" />',
			esc_attr( self::OPTION_NAME_CLIENT_ID ),
			esc_attr( self::OPTION_NAME_CLIENT_ID ),
			get_option( self::OPTION_NAME_CLIENT_ID )
		);
	}


	/**
	 * Audience field callback function
	 *
	 * @param array $args The field args
	 */
	public function audience_field_callback( $args ) {
		printf(
			'<input type="text" class="widefat" id="%s" name="%s" value="%s" />',
			esc_attr( self::OPTION_NAME_AUDIENCE ),
			esc_attr( self::OPTION_NAME_AUDIENCE ),
			get_option( self::OPTION_NAME_AUDIENCE )
		);
	}

	
	/**
	 * Secret field callback function
	 *
	 * @param array $args The field args
	 */
	public function secret_field_callback( $args ) {
		printf(
			'<input type="password" class="widefat" id="%s" name="%s" value="%s" />',
			esc_attr( self::OPTION_NAME_SECRET ),
			esc_attr( self::OPTION_NAME_SECRET ),
			get_option( self::OPTION_NAME_SECRET )
		);
	}

	/**
	 * Ingest the latest posts
	 */
	public function ingest_posts() {

		// Build the priorities array to query all priorities except 1.
		// Priority 1 is for alerts, which have no body attached.
		$priorities = array();
		for ( $i = 2; $i <= 9; $i++ ) {
			$priorities[] = "priority:${i}";
		}

		// Make the request with JSON payload
		$request = wp_remote_post(
			self::API_ENDPOINT,
			array(
				'headers' => array(
					'Authorization' => 'Bearer ' . self::get_auth_token(),
					'Content-Type'  => 'application/json'
				),
				'body' => json_encode(
					array(
						'query' => 'query MyQuery {
							search(query: "' . implode( '||', $priorities ) . '", filter: {mediaTypes: TEXT}) {
								totalHits
								items {
									uri
								}
							}
						}'
					)
				)
			)
		);

		$response = wp_remote_retrieve_body( $request );
		if ( empty( $response ) ) {
			return;
		}

		$body = json_decode( $response );
		
		if ( empty( $body->data->search->items )
			|| ! is_array( $body->data->search->items )
		) {
			return;
		}

		foreach ( $body->data->search->items as $item ) {

			// Check if post has already been ingested
			if ( empty( $item->uri )
				|| $this->post_exists( $item->uri )
			) {
				continue;
			}

			$request = wp_remote_post(
				self::API_ENDPOINT,
				array(
					'headers' => array(
						'Authorization' => 'Bearer ' . self::get_auth_token(),
						'Content-Type'  => 'application/json'
					),
					'body' => json_encode(
						array(
							'query' => 'query MyQuery {
								item(id: "' . $item->uri . '") {
									headLine
									bodyXhtml
									contentTimestamp
									subject {
										name
									}
								}
							}'
						)
					)
				)
			);

			$response = wp_remote_retrieve_body( $request );
			if ( empty( $response ) ) {
				continue;
			}

			$json = json_decode( $response );

			if ( empty( $json->data->item->bodyXhtml )
				|| empty( $json->data->item->contentTimestamp )
			) {
				continue;
			}

			// Create a DOMDocument so we can pluck out paragraphs
			$domdocument = new \DOMDocument();
			$domdocument->loadHTML( $json->data->item->bodyXhtml );
			$paragraphs = $domdocument->getElementsByTagName( 'p' );

			// Build a text string
			$text_content = '';
			foreach ( $paragraphs as $p ) {
				$text_content .= str_replace( "\n", ' ', $p->textContent ) . "\n\n";
			}

			$pieces = explode( "\n\n", $text_content );

			// If excerpt turns out to be very long, truncate to 100 words
			$excerpt = $pieces[0];
			if ( str_word_count( $excerpt ) > 1000 ) {
				$excerpt = wp_trim_words( $excerpt, 100 );
			}

			// Create new post object
			$post_id = wp_insert_post(
				array(
					'post_type'    => \DMG\Wires\Post_Type::NAME,
					'post_title'   => ( ! empty( $json->data->item->headLine  ) ) ? $json->data->item->headLine  : "[UNTITLED] Reuters ${item->uri}",
					'post_content' => $this->get_block_string_from_text( $text_content ),
					'post_excerpt' => $excerpt,
					'post_date'    => ( ! empty( $json->data->item->contentTimestamp ) ) ? $json->data->item->contentTimestamp : null,
					'post_status'  => 'pending',
				)
			);

			// If successful, set some additional info
			if ( ! is_wp_error( $post_id ) ) {

				// Set the source ID
				$this->set_source_id( $post_id, $item->uri );

				// Set source term
				$this->set_source_term( $post_id, self::SOURCE_NAME );

				// Set the word count
				$this->set_word_count( $post_id, str_word_count( $text_content ) );

				// Loop through all subjects and map them to existing tags/categories
				if ( ! empty( $json->data->item->subject )
					&& is_array( $json->data->item->subject )
				) {
					$tag_ids      = array();
					$category_ids = array();

					// Filter out nulls
					$raw_subjects = array_filter(
						$json->data->item->subject,
						function( $subject ) {
							return ( ! empty( $subject->name ) );
						}
					);

					// Create a clean array of subjects
					$subjects = array_map(
						function( $subject ) {
							return preg_replace( '/\(TRBC level \d+\)/', '', $subject->name );
						},
						$raw_subjects
					);

					foreach ( $subjects as $subject ) {

						$tag = get_term_by( 'name', $subject, 'post_tag' );
						if ( is_a( $tag, 'WP_Term' ) ) {
							$tag_ids[] = $tag->term_id;
						}

						$category = get_term_by( 'name', $subject, 'category' );
						if ( is_a( $category, 'WP_Term' ) ) {
							$category_ids[] = $category->term_id;
						}
					}

					if ( ! empty( $tag_ids ) ) {
						wp_set_object_terms( $post_id, $tag_ids, 'post_tag', true );
					}

					if ( ! empty( $category_ids ) ) {
						wp_set_object_terms( $post_id, $category_ids, 'category', true );
					}

					/**
					 * Fire off an action after terms are set
					 * 
					 * @param int $post_id The post ID
					 * @param array $subjects The subjects from the API response
					 */
					do_action( 'dmg_wires_reuters_after_subjects_set', $post_id, $subjects );
				}

				// Perform an action after the wire was created
				$this->do_action_after_create_post( $post_id );
			}
		}
	}

	/**
	 * Get the authentication token
	 * 
	 * @return string The auth token
	 */
	public static function get_auth_token() {

		// Get auth token from option
		$auth_token = get_option( self::OPTION_NAME_AUTH_TOKEN );

		// If empty, set one and return it
		if ( empty( $auth_token ) ) {
			$auth_token = self::set_auth_token();
		}

		return $auth_token;
	}

	/**
	 * Set the authentication token
	 * 
	 * @return string The auth token
	 */
	public static function set_auth_token() {

		$client_id = get_option( self::OPTION_NAME_CLIENT_ID );
		if ( empty( $client_id ) ) {
			return false;
		}

		$audience = get_option( self::OPTION_NAME_AUDIENCE );
		if ( empty( $audience ) ) {
			return false;
		}

		$secret = get_option( self::OPTION_NAME_SECRET );
		if ( empty( $secret ) ) {
			return false;
		}

		// Make the request with JSON payload
		$request = wp_remote_post(
			self::AUTH_ENDPOINT,
			array(
				'headers' => array(
					'Content-Type' => 'application/json'
				),
				'body' => json_encode(
					array(
						'client_id'     => $client_id,
						'audience'      => $audience,
						'client_secret' => $secret,
						'grant_type'    => 'client_credentials',
					)
				),
			)
		);

		$response = wp_remote_retrieve_body( $request );
		if ( empty( $response ) ) {
			return false;
		}

		$body = json_decode( $response );
		if ( ! empty( $body->access_token ) ) {
			update_option( self::OPTION_NAME_AUTH_TOKEN, $body->access_token );
			return $body->access_token;
		}

		return false;
	}

	/**
	 * Refresh credentials
	 * 
	 * Access tokens last only 24 hours so let's refresh them twice daily
	 */
	public function refresh_credentials() {
		self::set_auth_token();
	}

}
